// Copyright 2019 github.com. All rights reserved.
// Use of this source code is governed by github.com.

package v1

import (
	"git.getensh.com/common/gopkgs/logger"
	"git.getensh.com/common/gopkgs/tasker/httptasker"
	"git.getensh.com/common/gopkgs/util"
	"github.com/gin-gonic/gin"
	"go.uber.org/zap"
	"net/http"
	"property-company-gateway/errors"
	param_v1 "property-company-gateway/param/v1"
	"property-company-gateway/parser"
	"property-company-gateway/pb"
	"property-company-gateway/pb/v1"
	"property-company-gateway/utils"
)

//
// @Summary  注册公司
// @Description 注册公司
// @Tags 公司
// @Accept  json
// @Produce  json
// @Param body body v1.CompanyRegisterBody true "注册信息"
// @Success 200 {object} v1.CompanyRegisterResponse
// @Failure 500 {object} base.HTTPError
// @Router /api/v1/company [post]
func (c *Controller) CompanyRegister(ctx *gin.Context) {
	// 解析参数
	req := &param_v1.CompanyRegisterRequest{}
	parseParamTask := func() error {
		err := util.ShouldBind(ctx, nil, nil, nil, &req.CompanyRegisterBody)
		if err != nil {
			logger.Error("func",
				zap.String("call", "util.ShouldBind"),
				zap.String("error", err.Error()))
			return errors.ParamsError
		}
		return nil
	}

	// 业务处理
	handleServiceTask := func() error {
		// 响应数据
		resp := param_v1.CompanyRegisterResponse{}
		rpcReq := &v1.RegisterRequest{
			Applicant:               req.Applicant,
			ApplicantPhone:          req.ApplicantPhone,
			CompanyName:             req.CompanyName,
			CompanyPhone:            req.CompanyPhone,
			ApplicantDuty:           req.ApplicantDuty,
			LegalPerson:             req.LegalPerson,
			CompanyAddr:             req.CompanyAddr,
			SocialCode:              req.SocialCode,
			BusinessLicense:         req.BusinessLicense,
			ApplicantIdentification: req.ApplicantIdentification,
			LicenseType:             req.LicenseType,
			FreeGardenCount:         req.FreeGardenCount,
			Username:                req.Username,
			Password:                req.Password,
			//Expire:req.Expire,
			IsByManager: false,
			Site:        req.Site,
			Logo:        req.Logo,
			Desc:        req.Desc,
		}

		rpcRsp, err := pb.Company.Register(ctx, rpcReq)

		if err != nil {
			s, _ := json.MarshalToString(req)
			logger.Error("func",
				zap.String("call", "pb.Company.Register"),
				zap.String("params", s),
				zap.String("error", err.Error()))

			return errors.ErrorTransForm(err)
		}
		resp.Data = *rpcRsp
		ctx.JSON(http.StatusOK, resp)
		return nil
	}

	// 执行任务
	httptasker.Exec(ctx, parseParamTask, handleServiceTask)
}

//
// @Summary  更改公司信息
// @Description 更改公司信息
// @Tags 公司
// @Accept  json
// @Produce  json
// @Param token header string true " "
// @Param body body v1.CompanyUpdateBody true "注册信息"
// @Success 200 {object} v1.CompanyUpdateResponse
// @Failure 500 {object} base.HTTPError
// @Router /api/v1/company [put]
func (c *Controller) CompanyUpdate(ctx *gin.Context) {
	// 解析参数
	req := &param_v1.CompanyUpdateRequest{}
	parseParamTask := func() error {
		err := util.ShouldBind(ctx, nil, nil, nil, &req.CompanyUpdateBody)
		if err != nil {
			logger.Error("func",
				zap.String("call", "util.ShouldBind"),
				zap.String("error", err.Error()))
			return errors.ParamsError
		}
		return nil
	}

	// 业务处理
	handleServiceTask := func() error {
		tokenInfo, err := utils.GetJwtTokenInfo(ctx)
		if err != nil {
			return err
		}
		// 响应数据
		resp := param_v1.CompanyUpdateResponse{}
		rpcReq := &v1.CompanyUpdateRequest{
			CompanyName:     req.CompanyName,
			CompanyPhone:    req.CompanyPhone,
			CompanyAddr:     req.CompanyAddr,
			SocialCode:      req.SocialCode,
			BusinessLicense: req.BusinessLicense,
			LicenseType:     req.LicenseType,
			Site:            req.Site,
			Logo:            req.Logo,
			Desc:            req.Desc,
			Cid:             tokenInfo.Cid,
		}

		rpcRsp, err := pb.Company.CompanyUpdate(ctx, rpcReq)

		if err != nil {
			s, _ := json.MarshalToString(req)
			logger.Error("func",
				zap.String("call", "pb.Company.CompanyUpdate"),
				zap.String("params", s),
				zap.String("error", err.Error()))

			return errors.ErrorTransForm(err)
		}
		ctx.JSON(http.StatusOK, resp)

		logReq := OperationLogRequest{
			Module:   ModuleCompany,
			Action:   ActionCompanyUpdate,
			Origin:   rpcRsp.Origin,
			Target:   req.CompanyUpdateBody,
			UserName: tokenInfo.User,
			Uid:      tokenInfo.Uid,
			Cid:      tokenInfo.Cid,
		}
		go OperationLogAdd(&logReq)
		return nil
	}

	// 执行任务
	httptasker.Exec(ctx, parseParamTask, handleServiceTask)
}

//
// @Summary  获取公司信息
// @Description 获取公司信息
// @Tags 公司
// @Accept  json
// @Produce  json
// @Param token header string true " "
// @Success 200 {object} v1.CompanyInfoResponse
// @Failure 500 {object} base.HTTPError
// @Router /api/v1/company [get]
func (c *Controller) CompanyInfo(ctx *gin.Context) {
	// 解析参数
	req := &param_v1.CompanyInfoRequest{}
	parseParamTask := func() error {
		err := util.ShouldBind(ctx, nil, nil, nil, nil)
		if err != nil {
			logger.Error("func",
				zap.String("call", "util.ShouldBind"),
				zap.String("error", err.Error()))
			return errors.ParamsError
		}
		return nil
	}

	// 业务处理
	handleServiceTask := func() error {
		tokenInfo, err := utils.GetJwtTokenInfo(ctx)
		if err != nil {
			return err
		}
		// 响应数据
		resp := param_v1.CompanyInfoResponse{}
		rpcReq := &v1.CompanyInfoRequest{
			Id: tokenInfo.Cid,
		}

		rpcRsp, err := pb.Company.CompanyInfo(ctx, rpcReq)

		if err != nil {
			s, _ := json.MarshalToString(req)
			logger.Error("func",
				zap.String("call", "pb.Company.CompanyInfo"),
				zap.String("params", s),
				zap.String("error", err.Error()))

			return errors.ErrorTransForm(err)
		}
		resp.Data = *rpcRsp
		ctx.JSON(http.StatusOK, resp)
		return nil
	}

	// 执行任务
	httptasker.Exec(ctx, parseParamTask, handleServiceTask)
}

//
// @Summary  微信商户资料提交
// @Description 微信商户资料提交
// @Tags 公司
// @Accept  json
// @Produce  json
// @Param token header string true " "
// @Param body body v1.CompanyWxAccountApplyBody true "信息"
// @Success 200 {object} v1.CompanyWxAccountApplyResponse
// @Failure 500 {object} base.HTTPError
// @Router /api/v1/company/wx_account [post]
func (c *Controller) CompanyWxAccountApply(ctx *gin.Context) {
	// 解析参数
	req := &param_v1.CompanyWxAccountApplyRequest{}
	parseParamTask := func() error {
		err := util.ShouldBind(ctx, nil, nil, nil, &req.CompanyWxAccountApplyBody)
		if err != nil {
			logger.Error("func",
				zap.String("call", "util.ShouldBind"),
				zap.String("error", err.Error()))
			return errors.ParamsError
		}
		return nil
	}

	// 业务处理
	handleServiceTask := func() error {
		tokenInfo, err := utils.GetJwtTokenInfo(ctx)
		if err != nil {
			return err
		}
		// 响应数据
		resp := param_v1.CompanyWxAccountApplyResponse{}
		rpcReq := &v1.CompanyWxAccountApplyRequest{
			Cid: tokenInfo.Cid,
		}

		subject := v1.CompanyWxSubjectInfo{}
		business := v1.CompanyWxBusinessInfo{}
		bank := v1.CompanyWxBankAccountInfo{}
		contact := v1.CompanyWxContactInfo{}
		bytes, _ := json.Marshal(req.SubjectInfo)
		_ = json.Unmarshal(bytes, &subject)
		bytes, _ = json.Marshal(req.BusinessInfo)
		_ = json.Unmarshal(bytes, &business)
		bytes, _ = json.Marshal(req.BankAccountInfo)
		_ = json.Unmarshal(bytes, &bank)
		bytes, _ = json.Marshal(req.ContactInfo)
		_ = json.Unmarshal(bytes, &contact)

		rpcReq.SubjectInfo = &subject
		rpcReq.BusinessInfo = &business
		rpcReq.BankAccountInfo = &bank
		rpcReq.ContactInfo = &contact

		_, err = pb.Company.CompanyWxAccountApply(ctx, rpcReq)

		if err != nil {
			s, _ := json.MarshalToString(req)
			logger.Error("func",
				zap.String("call", "pb.Company.CompanyWxAccountApply"),
				zap.String("params", s),
				zap.String("error", err.Error()))

			return errors.ErrorTransForm(err)
		}

		ctx.JSON(http.StatusOK, resp)
		return nil
	}

	// 执行任务
	httptasker.Exec(ctx, parseParamTask, handleServiceTask)
}

//
// @Summary  微信商户申请记录
// @Description 微信商户申请记录
// @Tags 公司
// @Accept  json
// @Produce  json
// @Param token header string true " "
// @Param page query int true " "
// @Param page_size query int true " "
// @Success 200 {object} v1.CompanyWxAccountListResponse
// @Failure 500 {object} base.HTTPError
// @Router /api/v1/company/wx_account [get]
func (c *Controller) CompanyWxAccountList(ctx *gin.Context) {
	// 解析参数
	req := &param_v1.CompanyWxAccountListRequest{}
	parseParamTask := func() error {
		err := util.ShouldBind(ctx, nil, nil, &req.CompanyWxAccountListQuery, nil)
		if err != nil {
			logger.Error("func",
				zap.String("call", "util.ShouldBind"),
				zap.String("error", err.Error()))
			return errors.ParamsError
		}
		return nil
	}

	// 业务处理
	handleServiceTask := func() error {
		tokenInfo, err := utils.GetJwtTokenInfo(ctx)
		if err != nil {
			return err
		}
		if tokenInfo.Cid == 0 {
			return errors.ParamsError
		}
		// 响应数据
		resp := param_v1.CompanyWxAccountListResponse{}
		rpcReq := &v1.CompanyWxAccountApplyListRequest{
			Cid:      tokenInfo.Cid,
			PageSize: req.PageSize,
			Page:     req.Page,
		}

		rpcRsp, err := pb.Company.CompanyWxAccountApplyList(ctx, rpcReq)

		if err != nil {
			s, _ := json.MarshalToString(req)
			logger.Error("func",
				zap.String("call", "pb.Company.CompanyWxAccountList"),
				zap.String("params", s),
				zap.String("error", err.Error()))

			return errors.ErrorTransForm(err)
		}
		if rpcRsp.List == nil {
			rpcRsp.List = make([]*v1.CompanyWxAccountApplyListItem, 0)
		}
		resp.Data = *rpcRsp
		ctx.JSON(http.StatusOK, resp)
		return nil
	}

	// 执行任务
	httptasker.Exec(ctx, parseParamTask, handleServiceTask)
}

//
// @Summary  微信商户申请资料详情
// @Description 微信商户申请资料详情
// @Tags 公司
// @Accept  json
// @Produce  json
// @Param token header string true " "
// @Param id query int true "申请记录id"
// @Success 200 {object} v1.CompanyWxAccountApplyInfoResponse
// @Failure 500 {object} base.HTTPError
// @Router /api/v1/company/wx_account_info [get]
func (c *Controller) CompanyWxAccountApplyInfo(ctx *gin.Context) {
	// 解析参数
	req := &param_v1.CompanyWxAccountApplyInfoRequest{}
	parseParamTask := func() error {
		err := util.ShouldBind(ctx, nil, nil, &req.CompanyWxAccountApplyInfoQuery, nil)
		if err != nil {
			logger.Error("func",
				zap.String("call", "util.ShouldBind"),
				zap.String("error", err.Error()))
			return errors.ParamsError
		}
		return nil
	}

	// 业务处理
	handleServiceTask := func() error {
		tokenInfo, err := utils.GetJwtTokenInfo(ctx)
		if err != nil {
			return err
		}
		if tokenInfo.Cid == 0 {
			return errors.ParamsError
		}
		// 响应数据
		resp := param_v1.CompanyWxAccountApplyInfoResponse{}
		rpcReq := &v1.CompanyWxAccountApplyInfoRequest{
			Cid: tokenInfo.Cid,
			Id:  req.Id,
		}

		rpcRsp, err := pb.Company.CompanyWxAccountApplyInfo(ctx, rpcReq)

		if err != nil {
			s, _ := json.MarshalToString(req)
			logger.Error("func",
				zap.String("call", "pb.Company.CompanyWxAccountApplyInfo"),
				zap.String("params", s),
				zap.String("error", err.Error()))

			return errors.ErrorTransForm(err)
		}
		if rpcRsp.SubjectInfo.UboInfos == nil {
			rpcRsp.SubjectInfo.UboInfos = make([]*v1.CompanyWxIdDocInfo, 0)
		}
		resp.Data = *rpcRsp
		ctx.JSON(http.StatusOK, resp)
		return nil
	}

	// 执行任务
	httptasker.Exec(ctx, parseParamTask, handleServiceTask)
}

//
// @Summary  设置小区支付商户号
// @Description 设置小区支付商户号
// @Tags 公司
// @Accept  json
// @Produce  json
// @Param token header string true " "
// @Param body body v1.GardenSetMchBody true "信息"
// @Success 200 {object} v1.GardenSetMchResponse
// @Failure 500 {object} base.HTTPError
// @Router /api/v1/company/garden_mch [put]
func (c *Controller) GardenSetMch(ctx *gin.Context) {
	// 解析参数
	req := &param_v1.GardenSetMchRequest{}
	parseParamTask := func() error {
		err := util.ShouldBind(ctx, nil, nil, nil, &req.GardenSetMchBody)
		if err != nil {
			logger.Error("func",
				zap.String("call", "util.ShouldBind"),
				zap.String("error", err.Error()))
			return errors.ParamsError
		}
		return nil
	}

	// 业务处理
	handleServiceTask := func() error {
		tokenInfo, err := utils.GetJwtTokenInfo(ctx)
		if err != nil {
			return err
		}
		// 响应数据
		resp := param_v1.GardenSetMchResponse{}
		rpcReq := &v1.GardenSetMchRequest{
			Cid:      tokenInfo.Cid,
			GardenId: req.GardenId,
			MchId:    req.MchId,
		}

		_, err = pb.System.GardenSetMch(ctx, rpcReq)

		if err != nil {
			s, _ := json.MarshalToString(req)
			logger.Error("func",
				zap.String("call", "pb.System.GardenSetMch"),
				zap.String("params", s),
				zap.String("error", err.Error()))

			return errors.ErrorTransForm(err)
		}

		ctx.JSON(http.StatusOK, resp)
		return nil
	}

	// 执行任务
	httptasker.Exec(ctx, parseParamTask, handleServiceTask)
}

//
// @Summary  设置小区线上支付类型
// @Description 设置小区线上支付类型
// @Tags 公司
// @Accept  json
// @Produce  json
// @Param token header string true " "
// @Param body body v1.GardenSetMchPayModeBody true "信息"
// @Success 200 {object} v1.GardenSetMchPayModeResponse
// @Failure 500 {object} base.HTTPError
// @Router /api/v1/company/garden_paymode [put]
func (c *Controller) GardenSetMchPayMode(ctx *gin.Context) {
	// 解析参数
	req := &param_v1.GardenSetMchPayModeRequest{}
	parseParamTask := func() error {
		err := util.ShouldBind(ctx, nil, nil, nil, &req.GardenSetMchPayModeBody)
		if err != nil {
			logger.Error("func",
				zap.String("call", "util.ShouldBind"),
				zap.String("error", err.Error()))
			return errors.ParamsError
		}
		return nil
	}

	// 业务处理
	handleServiceTask := func() error {
		tokenInfo, err := utils.GetJwtTokenInfo(ctx)
		if err != nil {
			return err
		}
		// 响应数据
		resp := param_v1.GardenSetMchPayModeResponse{}
		rpcReq := &v1.GardenSetMchPayModeRequest{
			Cid:      tokenInfo.Cid,
			GardenId: req.GardenId,
			PayMode:  req.PayMode,
		}

		_, err = pb.System.GardenSetMchPayMode(ctx, rpcReq)

		if err != nil {
			s, _ := json.MarshalToString(req)
			logger.Error("func",
				zap.String("call", "pb.System.GardenSetMchPayMode"),
				zap.String("params", s),
				zap.String("error", err.Error()))

			return errors.ErrorTransForm(err)
		}

		ctx.JSON(http.StatusOK, resp)
		return nil
	}

	// 执行任务
	httptasker.Exec(ctx, parseParamTask, handleServiceTask)
}

//
// @Summary  获取小区支付商户号和支付类型信息
// @Description 获取小区支付商户号和支付类型信息
// @Tags 公司
// @Accept  json
// @Produce  json
// @Param token header string true " "
// @Param mch_id query string false "商户号 非必填"
// @Success 200 {object} v1.CompanyMchGardenListResponse
// @Failure 500 {object} base.HTTPError
// @Router /api/v1/company/garden_mch [get]
func (c *Controller) CompanyMchGardenList(ctx *gin.Context) {
	// 解析参数
	req := &param_v1.CompanyMchGardenListRequest{}
	parseParamTask := func() error {
		err := util.ShouldBind(ctx, nil, nil, &req.CompanyMchGardenListQuery, nil)
		if err != nil {
			logger.Error("func",
				zap.String("call", "util.ShouldBind"),
				zap.String("error", err.Error()))
			return errors.ParamsError
		}
		return nil
	}

	// 业务处理
	handleServiceTask := func() error {
		tokenInfo, err := utils.GetJwtTokenInfo(ctx)
		if err != nil {
			return err
		}
		// 响应数据
		resp := param_v1.CompanyMchGardenListResponse{}
		rpcReq := &v1.CompanyMchGardenListRequest{
			Cid:   tokenInfo.Cid,
			MchId: req.MchId,
		}

		rpcRsp, err := pb.System.CompanyMchGardenList(ctx, rpcReq)

		if err != nil {
			s, _ := json.MarshalToString(req)
			logger.Error("func",
				zap.String("call", "pb.System.CompanyMchGardenList"),
				zap.String("params", s),
				zap.String("error", err.Error()))

			return errors.ErrorTransForm(err)
		}
		if rpcRsp.List == nil {
			rpcRsp.List = make([]*v1.CompanyMchGardenItem, 0)
		}
		resp.Data = *rpcRsp

		ctx.JSON(http.StatusOK, resp)
		return nil
	}

	// 执行任务
	httptasker.Exec(ctx, parseParamTask, handleServiceTask)
}

//
// @Summary 获取静态文件url
// @Description 获取静态文件url
// @Tags 公司
// @Accept  json
// @Produce  json
// @Param token header string true "token"
// @Param template_type query int true "1 业务办理授权函 2 银行对照表 3 省市区编码对照表"
// @Success 200 {object} v1.TemplateResponse
// @Failure 500 {object} base.HTTPError
// @Router /api/v1/company/template [get]
func (c *Controller) CompanyTemplate(ctx *gin.Context) {
	// 解析参数
	const (
		businessLetter = 1
		bank           = 2
		province       = 3
	)
	req := &param_v1.TemplateRequest{}
	parseParamTask := func() error {
		err := util.ShouldBind(ctx, &req.Header, nil, &req.TemplateQuery, nil)
		if err != nil {
			logger.Error("func",
				zap.String("call", "util.ShouldBind"),
				zap.String("error", err.Error()))
			return errors.ParamsError
		}
		return nil
	}

	// 业务处理
	handleServiceTask := func() error {

		// 响应数据
		resp := param_v1.TemplateResponse{}
		switch req.TemplateType {
		case businessLetter:
			resp.Data.Url = parser.Conf.Oss.BusinessAuthorizationLetterTempUrl
		case bank:
			resp.Data.Url = parser.Conf.Oss.BankTempUrl
		case province:
			resp.Data.Url = parser.Conf.Oss.ProvinceTempUrl
		}
		ctx.JSON(http.StatusOK, resp)

		return nil
	}

	// 执行任务
	httptasker.Exec(ctx, parseParamTask, handleServiceTask)
}

//
// @Summary 统计
// @Description 统计
// @Tags 首页
// @Accept  json
// @Produce  json
// @Param token header string true "token"
// @Success 200 {object} v1.CompanyStatisticResponse
// @Failure 500 {object} base.HTTPError
// @Router /api/v1/statistic [get]
func (c *Controller) CompanyStatistic(ctx *gin.Context) {
	// 解析参数
	req := &param_v1.CompanyStatisticRequest{}
	parseParamTask := func() error {
		err := util.ShouldBind(ctx, &req.Header, nil, nil, nil)
		if err != nil {
			logger.Error("func",
				zap.String("call", "util.ShouldBind"),
				zap.String("error", err.Error()))
			return errors.ParamsError
		}
		return nil
	}

	// 业务处理
	handleServiceTask := func() error {
		tokenInfo, err := utils.GetJwtTokenInfo(ctx)
		if err != nil {
			return err
		}
		// 响应数据
		resp := param_v1.CompanyStatisticResponse{}
		rpcReq := &v1.CompanyStatisticRequest{
			Cid: tokenInfo.Cid,
		}

		rpcRsp, err := pb.Company.CompanyStatistic(ctx, rpcReq)

		if err != nil {
			s, _ := json.MarshalToString(req)
			logger.Error("func",
				zap.String("call", "pb.Company.CompanyStatistic"),
				zap.String("params", s),
				zap.String("error", err.Error()))

			return errors.ErrorTransForm(err)
		}

		resp.Data = *rpcRsp
		ctx.JSON(http.StatusOK, resp)

		return nil
	}

	// 执行任务
	httptasker.Exec(ctx, parseParamTask, handleServiceTask)
}
