// Copyright 2019 github.com. All rights reserved.
// Use of this source code is governed by github.com.

package v1

import (
	"fmt"
	"git.getensh.com/common/gopkgs/jwtwrapper"
	"git.getensh.com/common/gopkgs/logger"
	"git.getensh.com/common/gopkgs/tasker/httptasker"
	"git.getensh.com/common/gopkgs/util"
	"github.com/gin-gonic/gin"
	"go.uber.org/zap"
	"net/http"
	"property-company-gateway/errors"
	param_v1 "property-company-gateway/param/v1"
	"property-company-gateway/parser"
	"property-company-gateway/pb"
	"property-company-gateway/pb/v1"
	"property-company-gateway/utils"
	"time"
)

//
// @Summary 新增小区
// @Description 新增小区
// @Tags 小区
// @Accept  json
// @Produce  json
// @Param token header string true " "
// @Param body body v1.CompanyAddGardenBody true "小区信息"
// @Success 200 {object} v1.CompanyAddGardenResponse
// @Failure 500 {object} base.HTTPError
// @Router /api/v1/garden [post]
func (c *Controller) CompanyAddGarden(ctx *gin.Context) {
	// 解析参数
	req := &param_v1.CompanyAddGardenRequest{}
	parseParamTask := func() error {
		err := util.ShouldBind(ctx, &req.Header, nil, nil, &req.CompanyAddGardenBody)
		if err != nil {
			logger.Error("func",
				zap.String("call", "util.ShouldBind"),
				zap.String("error", err.Error()))
			return errors.ParamsError
		}
		return nil
	}

	// 业务处理
	handleServiceTask := func() error {
		// 响应数据
		tokenInfo, err := utils.GetJwtTokenInfo(ctx)
		if err != nil {
			return err
		}
		resp := param_v1.CompanyAddGardenResponse{}
		rpcReq := &v1.CompanyAddGardenRequest{
			Province:       req.Province,
			ProvinceCode:   req.ProvinceCode,
			City:           req.City,
			CityCode:       req.CityCode,
			Area:           req.Area,
			AreaCode:       req.AreaCode,
			Street:         req.Street,
			StreetCode:     req.StreetCode,
			Committee:      req.Committee,
			CommitteeCode:  req.CommitteeCode,
			PropertyPerson: req.PropertyPerson,
			PropertyPhone:  req.PropertyPhone,
			GardenName:     req.GardenName,
			GardenAddr:     req.GardenAddr,
			Cid:            tokenInfo.Cid,
			GardenDesc:     req.GardenDesc,
			GardenPic:      req.GardenPic,
			Lat:            req.Lat,
			Lnt:            req.Lnt,
			Appendix:       req.Appendix,
			OrderId:        req.OrderId,

			GardenPics:       req.GardenPics,
			BuildingStart:    req.BuildingStart,
			BuildingEnd:      req.BuildingEnd,
			PropertyFeeStart: req.PropertyFeeStart,
			PropertyFeeEnd:   req.PropertyFeeEnd,
			GasFeeStart:      req.GasFeeStart,
			GasFeeEnd:        req.GasFeeEnd,
			BuildingArea:     req.BuildingArea,
			BuildingCompany:  req.BuildingCompany,
			CoveredArea:      req.CoveredArea,
			GreenPercent:     req.GreenPercent,
			AreaPercent:      req.AreaPercent,
			SpaceInfo:        req.SpaceInfo,
			SpaceTotal:       req.SpaceTotal,
			HouseTotal:       req.HouseTotal,
			BuildingType:     req.BuildingType,
			WaterType:        req.WaterType,
			ElectricType:     req.ElectricType,
			AvgPrice:         req.AvgPrice,
		}

		rpcRsp, err := pb.Company.CompanyAddGarden(ctx, rpcReq)

		if err != nil {
			s, _ := json.MarshalToString(req)
			logger.Error("func",
				zap.String("call", "pb.Management.CompanyAddGarden"),
				zap.String("params", s),
				zap.String("error", err.Error()))

			return errors.ErrorTransForm(err)
		}

		resp.Data = *rpcRsp
		ctx.JSON(http.StatusOK, resp)

		logReq := OperationLogRequest{
			Module:   ModuleGarden,
			Action:   ActionGardenAdd,
			Origin:   nil,
			Target:   req.CompanyAddGardenBody,
			UserName: tokenInfo.User,
			Uid:      tokenInfo.Uid,
			Cid:      tokenInfo.Cid,
		}
		go OperationLogAdd(&logReq)
		return nil
	}

	// 执行任务
	httptasker.Exec(ctx, parseParamTask, handleServiceTask)
}

//
// @Summary 小区列表
// @Description 小区列表
// @Tags 小区
// @Accept  json
// @Produce  json
// @Param token header string true " "
// @Param page query int true " "
// @Param page_size query int false " "
// @Param not_approved query bool false "true 待审批或审批未通过的数据 false 审批通过的数据 "
// @Success 200 {object} v1.CompanyGardenListResponse
// @Failure 500 {object} base.HTTPError
// @Router /api/v1/garden [get]
func (c *Controller) CompanyGardenList(ctx *gin.Context) {
	// 解析参数
	req := &param_v1.CompanyGardenListRequest{}
	parseParamTask := func() error {
		err := util.ShouldBind(ctx, &req.Header, nil, &req.CompanyGardenListQuery, nil)
		if err != nil {
			logger.Error("func",
				zap.String("call", "util.ShouldBind"),
				zap.String("error", err.Error()))
			return errors.ParamsError
		}
		return nil
	}

	// 业务处理
	handleServiceTask := func() error {
		// 响应数据
		tokenInfo, err := utils.GetJwtTokenInfo(ctx)
		if err != nil {
			return err
		}
		resp := param_v1.CompanyGardenListResponse{}
		rpcReq := &v1.GardenListRequest{
			Page:        req.Page,
			PageSize:    req.PageSize,
			Cid:         tokenInfo.Cid,
			NotApproved: req.NotApproved,
			GardenName:  req.GardenName,
		}
		//fmt.Printf("xxxxxxxxxxxxx:%v\n", rpcReq.NotApproved)

		rpcRsp, err := pb.System.GardenList(ctx, rpcReq)

		if err != nil {
			s, _ := json.MarshalToString(req)
			logger.Error("func",
				zap.String("call", "Garden.GardenList"),
				zap.String("params", s),
				zap.String("error", err.Error()))

			return errors.ErrorTransForm(err)
		}

		for i, v := range rpcRsp.List {
			if len(v.WaterType) == 0 {
				rpcRsp.List[i].WaterType = []int32{}
			}
			if len(v.ElectricType) == 0 {
				rpcRsp.List[i].ElectricType = []int32{}
			}
			if len(v.GardenPics) == 0 {
				rpcRsp.List[i].GardenPics = []string{}
			}
		}

		if rpcRsp.List == nil {
			rpcRsp.List = make([]*v1.GardenItem, 0)
		}
		resp.Data = *rpcRsp

		ctx.JSON(http.StatusOK, resp)
		return nil
	}

	// 执行任务
	httptasker.Exec(ctx, parseParamTask, handleServiceTask)
}

//
// @Summary 判断是否可以添加小区
// @Description 判断是否可以添加小区
// @Tags 小区
// @Accept  json
// @Produce  json
// @Param token header string true " "
// @Success 200 {object} v1.CompanyCanAddGardenResponse
// @Failure 500 {object} base.HTTPError
// @Router /api/v1/garden/check [get]
func (c *Controller) CompanyCanAddGarden(ctx *gin.Context) {
	// 解析参数
	req := &param_v1.CompanyCanAddGardenRequest{}
	parseParamTask := func() error {
		err := util.ShouldBind(ctx, &req.Header, nil, nil, nil)
		if err != nil {
			logger.Error("func",
				zap.String("call", "util.ShouldBind"),
				zap.String("error", err.Error()))
			return errors.ParamsError
		}
		return nil
	}

	// 业务处理
	handleServiceTask := func() error {
		// 响应数据
		tokenInfo, err := utils.GetJwtTokenInfo(ctx)
		if err != nil {
			return err
		}
		resp := param_v1.CompanyCanAddGardenResponse{}
		rpcReq := &v1.CompanyCanAddGardenRequest{

			Cid: tokenInfo.Cid,
		}
		//fmt.Printf("xxxxxxxxxxxxx:%v\n", rpcReq.NotApproved)

		rpcRsp, err := pb.Company.CompanyCanAddGarden(ctx, rpcReq)

		if err != nil {
			s, _ := json.MarshalToString(req)
			logger.Error("func",
				zap.String("call", "CompanyCanAddGarden"),
				zap.String("params", s),
				zap.String("error", err.Error()))

			return errors.ErrorTransForm(err)
		}

		resp.Data = *rpcRsp

		ctx.JSON(http.StatusOK, resp)
		return nil
	}

	// 执行任务
	httptasker.Exec(ctx, parseParamTask, handleServiceTask)
}

//
// @Summary 修改小区
// @Description 修改小区
// @Tags 小区
// @Accept  json
// @Produce  json
// @Param token header string true " "
// @Param body body v1.CompanyGardenUpdateBody true "小区信息"
// @Success 200 {object} v1.CompanyGardenUpdateResponse
// @Failure 500 {object} base.HTTPError
// @Router /api/v1/garden [put]
func (c *Controller) CompanyGardenUpdate(ctx *gin.Context) {
	// 解析参数
	req := &param_v1.CompanyGardenUpdateRequest{}
	parseParamTask := func() error {
		err := util.ShouldBind(ctx, &req.Header, nil, nil, &req.CompanyGardenUpdateBody)
		if err != nil {
			logger.Error("func",
				zap.String("call", "util.ShouldBind"),
				zap.String("error", err.Error()))
			return errors.ParamsError
		}
		return nil
	}

	// 业务处理
	handleServiceTask := func() error {
		// 响应数据
		tokenInfo, err := utils.GetJwtTokenInfo(ctx)
		if err != nil {
			return err
		}
		resp := param_v1.CompanyGardenUpdateResponse{}
		rpcReq := &v1.GardenUpdateRequest{
			//Province:       req.Province,
			//ProvinceCode:   req.ProvinceCode,
			//City:           req.City,
			//CityCode:       req.CityCode,
			//Area:           req.Area,
			//AreaCode:       req.AreaCode,
			//Street:         req.Street,
			//StreetCode:     req.StreetCode,
			//Committee:      req.Committee,
			//CommitteeCode:  req.CommitteeCode,
			PropertyPerson: req.PropertyPerson,
			PropertyPhone:  req.PropertyPhone,
			//GardenName:     req.GardenName,
			GardenAddr: req.GardenAddr,
			Cid:        tokenInfo.Cid,
			Id:         req.Id,
			GardenPic:  req.GardenPic,
			GardenDesc: req.GardenDesc,
			Lat:        req.Lat,
			Lnt:        req.Lnt,

			GardenPics:       req.GardenPics,
			BuildingStart:    req.BuildingStart,
			BuildingEnd:      req.BuildingEnd,
			PropertyFeeStart: req.PropertyFeeStart,
			PropertyFeeEnd:   req.PropertyFeeEnd,
			GasFeeStart:      req.GasFeeStart,
			GasFeeEnd:        req.GasFeeEnd,
			BuildingArea:     req.BuildingArea,
			BuildingCompany:  req.BuildingCompany,
			CoveredArea:      req.CoveredArea,
			GreenPercent:     req.GreenPercent,
			AreaPercent:      req.AreaPercent,
			SpaceInfo:        req.SpaceInfo,
			SpaceTotal:       req.SpaceTotal,
			HouseTotal:       req.HouseTotal,
			BuildingType:     req.BuildingType,
			WaterType:        req.WaterType,
			ElectricType:     req.ElectricType,
			AvgPrice:         req.AvgPrice,
		}

		rpcRsp, err := pb.System.GardenUpdate(ctx, rpcReq)

		if err != nil {
			s, _ := json.MarshalToString(req)
			logger.Error("func",
				zap.String("call", "pb.Garden.GardenUpdate"),
				zap.String("params", s),
				zap.String("error", err.Error()))

			return errors.ErrorTransForm(err)
		}
		ctx.JSON(http.StatusOK, resp)

		logReq := OperationLogRequest{
			Module:   ModuleGarden,
			Action:   ActionGardenUpdate,
			Origin:   rpcRsp.Origin,
			Target:   req.CompanyGardenUpdateBody,
			UserName: tokenInfo.User,
			Uid:      tokenInfo.Uid,
			Cid:      tokenInfo.Cid,
		}
		go OperationLogAdd(&logReq)
		return nil
	}

	// 执行任务
	httptasker.Exec(ctx, parseParamTask, handleServiceTask)
}

//
// @Summary 删除小区
// @Description 删除小区
// @Tags 小区
// @Accept  json
// @Produce  json
// @Param token header string true " "
// @Param id query int true "小区id"
// @Success 200 {object} v1.CompanyGardenDelResponse
// @Failure 500 {object} base.HTTPError
// @Router /api/v1/garden [delete]
func (c *Controller) CompanyGardenDel(ctx *gin.Context) {
	// 解析参数
	req := &param_v1.CompanyGardenDelRequest{}
	parseParamTask := func() error {
		err := util.ShouldBind(ctx, &req.Header, nil, &req.CompanyGardenDelQuery, nil)
		if err != nil {
			logger.Error("func",
				zap.String("call", "util.ShouldBind"),
				zap.String("error", err.Error()))
			return errors.ParamsError
		}
		return nil
	}

	// 业务处理
	handleServiceTask := func() error {
		// 响应数据
		tokenInfo, err := utils.GetJwtTokenInfo(ctx)
		if err != nil {
			return err
		}
		resp := param_v1.CompanyGardenDelResponse{}
		rpcReq := &v1.GardenDelRequest{
			Id:  req.Id,
			Cid: tokenInfo.Cid,
		}

		rpcRsp, err := pb.System.GardenDel(ctx, rpcReq)

		if err != nil {
			s, _ := json.MarshalToString(req)
			logger.Error("func",
				zap.String("call", "pb.Garden.GardenDel"),
				zap.String("params", s),
				zap.String("error", err.Error()))

			return errors.ErrorTransForm(err)
		}
		ctx.JSON(http.StatusOK, resp)

		logReq := OperationLogRequest{
			Module:   ModuleGarden,
			Action:   ActionGardenDel,
			Origin:   rpcRsp.Origin,
			Target:   nil,
			UserName: tokenInfo.User,
			Uid:      tokenInfo.Uid,
			Cid:      tokenInfo.Cid,
		}
		go OperationLogAdd(&logReq)
		return nil
	}

	// 执行任务
	httptasker.Exec(ctx, parseParamTask, handleServiceTask)
}

//
// @Summary 小区入口
// @Description 小区入口
// @Tags 小区
// @Accept  json
// @Produce  json
// @Param token header string true " "
// @Param garden_id query int true " "
// @Success 200 {object} v1.CompanyEnterGardenResponse
// @Failure 500 {object} base.HTTPError
// @Router /api/v1/garden/entry [get]
func (c *Controller) CompanyEnterGarden(ctx *gin.Context) {
	// 解析参数
	req := &param_v1.CompanyEnterGardenRequest{}
	parseParamTask := func() error {
		err := util.ShouldBind(ctx, &req.Header, nil, &req.CompanyEnterGardenQuery, nil)
		if err != nil {
			logger.Error("func",
				zap.String("call", "util.ShouldBind"),
				zap.String("error", err.Error()))
			return errors.ParamsError
		}
		return nil
	}

	// 业务处理
	handleServiceTask := func() error {
		// 响应数据
		tokenInfo, err := utils.GetJwtTokenInfo(ctx)
		if err != nil {
			return err
		}
		subject := map[string]interface{}{
			"user_name":        tokenInfo.User,
			"cid":              tokenInfo.Cid,
			"garden_id":        req.GardenId,
			"is_company_enter": true,
		}
		str, _ := json.MarshalToString(subject)

		// 生成token
		token, err := jwtwrapper.GenToken(fmt.Sprintf("%d", tokenInfo.Uid), parser.Conf.Jwt.Issuer, str,
			time.Duration(parser.Conf.Jwt.Seconds)*time.Second)
		if err != nil {
			logger.Error("func",
				zap.String("call", "util.GenJwtToken"),
				zap.String("args", fmt.Sprintf("%d", tokenInfo.Uid)),
				zap.String("error", err.Error()))
			return errors.SystemError
		}
		resp := param_v1.CompanyEnterGardenResponse{}
		resp.Data.Token = token
		ctx.JSON(http.StatusOK, resp)
		return nil
	}

	// 执行任务
	httptasker.Exec(ctx, parseParamTask, handleServiceTask)
}

//
// @Summary 申请修改小区关键信息
// @Description 申请修改小区关键信息
// @Tags 小区
// @Accept  json
// @Produce  json
// @Param token header string true " "
// @Param body body v1.GardenKeyInfoChangeBody true "小区信息"
// @Success 200 {object} v1.GardenKeyInfoChangeResponse
// @Failure 500 {object} base.HTTPError
// @Router /api/v1/garden/key_info [put]
func (c *Controller) GardenKeyInfoChange(ctx *gin.Context) {
	// 解析参数
	req := &param_v1.GardenKeyInfoChangeRequest{}
	parseParamTask := func() error {
		err := util.ShouldBind(ctx, &req.Header, nil, nil, &req.GardenKeyInfoChangeBody)
		if err != nil {
			logger.Error("func",
				zap.String("call", "util.ShouldBind"),
				zap.String("error", err.Error()))
			return errors.ParamsError
		}
		return nil
	}

	// 业务处理
	handleServiceTask := func() error {
		tokenInfo, err := utils.GetJwtTokenInfo(ctx)
		if err != nil {
			return err
		}
		// 响应数据
		resp := param_v1.GardenKeyInfoChangeResponse{}
		rpcReq := &v1.GardenKeyInfoChangeRequest{
			Province:      req.Province,
			ProvinceCode:  req.ProvinceCode,
			City:          req.City,
			CityCode:      req.CityCode,
			Area:          req.Area,
			AreaCode:      req.AreaCode,
			Street:        req.Street,
			StreetCode:    req.StreetCode,
			Committee:     req.Committee,
			CommitteeCode: req.CommitteeCode,
			Cid:           tokenInfo.Cid,
			Id:            req.GardenId,
			GardenName:    req.GardenName,
		}

		_, err = pb.System.GardenKeyInfoChange(ctx, rpcReq)

		if err != nil {
			s, _ := json.MarshalToString(req)
			logger.Error("func",
				zap.String("call", "pb.Garden.GardenKeyInfoChange"),
				zap.String("params", s),
				zap.String("error", err.Error()))

			return errors.ErrorTransForm(err)
		}

		ctx.JSON(http.StatusOK, resp)
		return nil
	}

	// 执行任务
	httptasker.Exec(ctx, parseParamTask, handleServiceTask)
}

//
// @Summary 小区关键信息申请列表
// @Description 小区关键信息申请列表
// @Tags 小区
// @Accept  json
// @Produce  json
// @Param token header string true " "
// @Param page query int false " "
// @Param page_size query int false " "
// @Param status query int false "0不过率 1 待审核 2 审核通过 3 未通过 "
// @Success 200 {object} v1.GardenKeyInfoChangeListResponse
// @Failure 500 {object} base.HTTPError
// @Router /api/v1/garden/key_info [get]
func (c *Controller) GardenKeyInfoChangeList(ctx *gin.Context) {
	// 解析参数
	req := &param_v1.GardenKeyInfoChangeListRequest{}
	parseParamTask := func() error {
		err := util.ShouldBind(ctx, &req.Header, nil, &req.GardenKeyInfoChangeListQuery, nil)
		if err != nil {
			logger.Error("func",
				zap.String("call", "util.ShouldBind"),
				zap.String("error", err.Error()))
			return errors.ParamsError
		}
		return nil
	}

	// 业务处理
	handleServiceTask := func() error {
		tokenInfo, err := utils.GetJwtTokenInfo(ctx)
		if err != nil {
			return err
		}
		// 响应数据
		resp := param_v1.GardenKeyInfoChangeListResponse{}
		rpcReq := &v1.GardenKeyInfoChangeListRequest{
			Cid:      tokenInfo.Cid,
			GardenId: 0,
			Status:   req.Status,
			Page:     req.Page,
			PageSize: req.PageSize,
		}

		rpcRsp, err := pb.System.GardenKeyInfoChangeList(ctx, rpcReq)

		if err != nil {
			s, _ := json.MarshalToString(req)
			logger.Error("func",
				zap.String("call", "pb.Garden.GardenKeyInfoChangeList"),
				zap.String("params", s),
				zap.String("error", err.Error()))

			return errors.ErrorTransForm(err)
		}

		if rpcRsp.List == nil {
			rpcRsp.List = make([]*v1.GardenKeyInfoData, 0)
		}
		resp.Data = *rpcRsp
		ctx.JSON(http.StatusOK, resp)
		return nil
	}

	// 执行任务
	httptasker.Exec(ctx, parseParamTask, handleServiceTask)
}
