// Copyright 2019 github.com. All rights reserved.
// Use of this source code is governed by github.com.

package v1

/*
//
// @Summary 新增用户
// @Description 新增用户
// @Tags 物业端管理人员
// @Accept  json
// @Produce  json
// @Param token header string true " "
// @Param body body v1.SystemUserAddBody true "小区信息"
// @Success 200 {object} v1.SystemUserAddResponse
// @Failure 500 {object} base.HTTPError
// @Router /api/v1/system/user [post]
func (c *Controller) SystemUserAdd(ctx *gin.Context) {
	// 解析参数
	req := &param_v1.SystemUserAddRequest{}
	superGroupId := int64(0)
	parseParamTask := func() error {
		err := util.ShouldBind(ctx, &req.Header, nil, nil, &req.SystemUserAddBody)
		if err != nil {
			logger.Error("func",
				zap.String("call", "util.ShouldBind"),
				zap.String("error", err.Error()))
			return errors.ParamsError
		}
		return nil
	}
	getSuperGroup := func() error {
		tokenInfo, err := utils.GetJwtTokenInfo(ctx)
		if err != nil {
			return err
		}
		mreq := &v1.SuperGroupRequest{
			GardenId:req.GardenId,
			Cid:tokenInfo.Cid,
		}
		mreply, err := pb.System.SuperGroup(ctx, mreq)
		if err != nil {
			s, _ := json.MarshalToString(req)
			logger.Error("func",
				zap.String("call", "pb.Management.CompanyAddGarden"),
				zap.String("params", s),
				zap.String("error", err.Error()))

			return errors.ErrorTransForm(err)
		}

		superGroupId = mreply.Id
		return nil
	}

	// 业务处理
	handleServiceTask := func() error {
		// 响应数据
		tokenInfo, err := utils.GetJwtTokenInfo(ctx)
		if err != nil {
			return err
		}
		resp := param_v1.SystemUserAddResponse{}
		rpcReq := &v1.UserAddRequest{
			UserName:req.UserName,
			Password:req.Password,
			Phone:req.Phone,
			Email:req.Email,
			GardenId:req.GardenId,
			GardenName:     req.GardenName,
			Cid:            tokenInfo.Cid,
			GroupId:superGroupId,
			Enable:req.Enable,
			ByCompany:true,
		}

		rpcRsp, err := pb.System.UserAdd(ctx, rpcReq)

		if err != nil {
			s, _ := json.MarshalToString(req)
			logger.Error("func",
				zap.String("call", "pb.System.UserAdd"),
				zap.String("params", s),
				zap.String("error", err.Error()))

			return errors.ErrorTransForm(err)
		}

		resp.Data = *rpcRsp
		ctx.JSON(http.StatusOK, resp)

		logReq := OperationLogRequest{
			Module:ModuleSystemUser,
			Action:ActionSystemUserAdd,
			Origin:nil,
			Target:req.SystemUserAddBody,
			UserName:tokenInfo.User,
			Uid:tokenInfo.Uid,
			Cid:tokenInfo.Cid,
		}
		go OperationLogAdd(&logReq)
		return nil
	}

	// 执行任务
	httptasker.Exec(ctx, parseParamTask, getSuperGroup, handleServiceTask)
}

//
// @Summary 修改用户
// @Description 修改用户
// @Tags 物业端管理人员
// @Accept  json
// @Produce  json
// @Param token header string true " "
// @Param body body v1.SystemUserUpdateBody true "小区信息"
// @Success 200 {object} v1.SystemUserUpdateResponse
// @Failure 500 {object} base.HTTPError
// @Router /api/v1/system/user [put]
func (c *Controller) SystemUserUpdate(ctx *gin.Context) {
	// 解析参数
	req := &param_v1.SystemUserUpdateRequest{}
	superGroupId := int64(0)
	parseParamTask := func() error {
		err := util.ShouldBind(ctx, &req.Header, nil, nil, &req.SystemUserUpdateBody)
		if err != nil {
			logger.Error("func",
				zap.String("call", "util.ShouldBind"),
				zap.String("error", err.Error()))
			return errors.ParamsError
		}
		return nil
	}

	getSuperGroup := func() error {
		tokenInfo, err := utils.GetJwtTokenInfo(ctx)
		if err != nil {
			return err
		}
		mreq := &v1.SuperGroupRequest{
			GardenId:req.GardenId,
			Cid:tokenInfo.Cid,
		}
		mreply, err := pb.System.SuperGroup(ctx, mreq)
		if err != nil {
			s, _ := json.MarshalToString(req)
			logger.Error("func",
				zap.String("call", "pb.Management.CompanyAddGarden"),
				zap.String("params", s),
				zap.String("error", err.Error()))

			return errors.ErrorTransForm(err)
		}

		superGroupId = mreply.Id
		return nil
	}


	// 业务处理
	handleServiceTask := func() error {
		// 响应数据
		tokenInfo, err := utils.GetJwtTokenInfo(ctx)
		if err != nil {
			return err
		}
		resp := param_v1.SystemUserUpdateResponse{}
		rpcReq := &v1.UserUpdateRequest{
			UserName:req.UserName,
			Password:req.Password,
			Phone:req.Phone,
			Email:req.Email,
			GardenId:req.GardenId,
			GardenName:     req.GardenName,
			Cid:            tokenInfo.Cid,
			Id:req.Id,
			Enable:req.Enable,
			GroupId:superGroupId,
			ByCompany:true,
		}
		rpcRsp, err := pb.System.UserUpdate(ctx, rpcReq)

		if err != nil {
			s, _ := json.MarshalToString(req)
			logger.Error("func",
				zap.String("call", "pb.System.UserUpdate"),
				zap.String("params", s),
				zap.String("error", err.Error()))

			return errors.ErrorTransForm(err)
		}
		ctx.JSON(http.StatusOK, resp)

		logReq := OperationLogRequest{
			Module:ModuleSystemUser,
			Action:ActionSystemUserUpdate,
			Origin:rpcRsp.Origin,
			Target:req.SystemUserUpdateBody,
			UserName:tokenInfo.User,
			Uid:tokenInfo.Uid,
			Cid:tokenInfo.Cid,
		}
		go OperationLogAdd(&logReq)
		return nil
	}

	// 执行任务
	httptasker.Exec(ctx, parseParamTask, getSuperGroup, handleServiceTask)
}

//
// @Summary 删除用户
// @Description 删除用户
// @Tags 物业端管理人员
// @Accept  json
// @Produce  json
// @Param token header string true " "
// @Param id query int true "物业用户id"
// @Success 200 {object} v1.SystemUserDelResponse
// @Failure 500 {object} base.HTTPError
// @Router /api/v1/system/user [delete]
func (c *Controller) SystemUserDel(ctx *gin.Context) {
	// 解析参数
	req := &param_v1.SystemUserDelRequest{}
	parseParamTask := func() error {
		err := util.ShouldBind(ctx, &req.Header, nil, &req.SystemUserDelQuery, nil)
		if err != nil {
			logger.Error("func",
				zap.String("call", "util.ShouldBind"),
				zap.String("error", err.Error()))
			return errors.ParamsError
		}
		return nil
	}

	// 业务处理
	handleServiceTask := func() error {
		tokenInfo, err := utils.GetJwtTokenInfo(ctx)
		if err != nil {
			return err
		}
		// 响应数据
		resp := param_v1.SystemUserDelResponse{}
		rpcReq := &v1.UserDelRequest{
			Id:req.Id,
			Cid:tokenInfo.Cid,
			ByCompany:true,

		}

		rpcRsp, err := pb.System.UserDel(ctx, rpcReq)

		if err != nil {
			s, _ := json.MarshalToString(req)
			logger.Error("func",
				zap.String("call", "pb.System.UserDel"),
				zap.String("params", s),
				zap.String("error", err.Error()))

			return errors.ErrorTransForm(err)
		}

		ctx.JSON(http.StatusOK, resp)

		logReq := OperationLogRequest{
			Module:ModuleSystemUser,
			Action:ActionSystemUserDel,
			Origin:rpcRsp.Origin,
			Target:nil,
			UserName:tokenInfo.User,
			Uid:tokenInfo.Uid,
			Cid:tokenInfo.Cid,
		}
		go OperationLogAdd(&logReq)
		return nil
	}

	// 执行任务
	httptasker.Exec(ctx, parseParamTask,  handleServiceTask)
}


//
// @Summary 用户列表
// @Description 用户列表
// @Tags 物业端管理人员
// @Accept  json
// @Produce  json
// @Param token header string true " "
// @Param page query int false "页号"
// @Param page_size query int false "一页记录，-1表示不分页获取所有"
// @Param garden_name query string false "小区名称，模糊搜索"
// @Param user_name query string false "用户名，模糊搜索"
// @Param garden_id query int false "小区id,用于获取某小区的物业用户"
// @Success 200 {object} v1.SystemUserListResponse
// @Failure 500 {object} base.HTTPError
// @Router /api/v1/system/user [get]
func (c *Controller) SystemUserList(ctx *gin.Context) {
	// 解析参数
	req := &param_v1.SystemUserListRequest{}
	parseParamTask := func() error {
		err := util.ShouldBind(ctx, &req.Header, nil, &req.SystemUserListQuery, nil)
		if err != nil {
			logger.Error("func",
				zap.String("call", "util.ShouldBind"),
				zap.String("error", err.Error()))
			return errors.ParamsError
		}
		return nil
	}

	// 业务处理
	handleServiceTask := func() error {
		// 响应数据
		tokenInfo, err := utils.GetJwtTokenInfo(ctx)
		if err != nil {
			return err
		}
		resp := param_v1.SystemUserListResponse{}
		rpcReq := &v1.UserListRequest{
			Cid:tokenInfo.Cid,
			GardenId:req.GardenId,
			GardenName:req.GardenName,
			UserName:req.UserName,
			PageSize:req.PageSize,
			Page:req.Page,
			ByCompany:true,
		}

		rpcRsp, err := pb.System.UserList(ctx, rpcReq)

		if err != nil {
			s, _ := json.MarshalToString(req)
			logger.Error("func",
				zap.String("call", "pb.System.UserList"),
				zap.String("params", s),
				zap.String("error", err.Error()))

			return errors.ErrorTransForm(err)
		}
		if rpcRsp.List == nil {
			rpcRsp.List = make([]*v1.UserItem, 0)
		}
		resp.Data = *rpcRsp

		ctx.JSON(http.StatusOK, resp)
		return nil
	}

	// 执行任务
	httptasker.Exec(ctx, parseParamTask,  handleServiceTask)
}

 */