// Copyright 2019 github.com. All rights reserved.
// Use of this source code is governed by github.com.

package v1

import (
	"fmt"
	"github.com/tidwall/gjson"
	"net/http"
	"property-company-gateway/errors"
	param_v1 "property-company-gateway/param/v1"
	"property-company-gateway/pb"
	"property-company-gateway/pb/v1"
	"property-company-gateway/utils"
	"time"

	"git.getensh.com/common/gopkgs/logger"
	"git.getensh.com/common/gopkgs/tasker/httptasker"
	"git.getensh.com/common/gopkgs/util"
	"github.com/dgrijalva/jwt-go"

	"property-company-gateway/parser"

	"git.getensh.com/common/gopkgs/jwtwrapper"
	"github.com/gin-gonic/gin"
	"go.uber.org/zap"
)

// 登录
// @Summary 登录
// @Description 登录
// @Tags 用户
// @Accept  json
// @Produce  json
// @Param body body v1.LoginBody true "登录信息"
// @Success 200 {object} v1.LoginResponse
// @Failure 500 {object} base.HTTPError
// @Router /api/v1/user/login [post]
func (c *Controller) Login(ctx *gin.Context) {
	// 解析参数
	req := &param_v1.LoginRequest{}
	parseParamTask := func() error {
		err := util.ShouldBind(ctx, &req.Header, nil, nil, &req.LoginBody)
		if err != nil {
			logger.Error("func",
				zap.String("call", "util.ShouldBind"),
				zap.String("error", err.Error()))
			return errors.ParamsError
		}
		return nil
	}

	// 业务处理
	handleServiceTask := func() error {
		// 响应数据
		resp := param_v1.LoginResponse{}
		rpcReq := &v1.LoginRequest{
			Username: req.User,
			Password: req.Password,
		}

		rpcRsp, err := pb.Company.Login(ctx, rpcReq)

		if err != nil {
			s, _ := json.MarshalToString(req)
			logger.Error("func",
				zap.String("call", "pb.Management.Login"),
				zap.String("params", s),
				zap.String("error", err.Error()))

			return errors.ErrorTransForm(err)
		}
		subject := map[string]interface{}{
			"user_name":    req.User,
			"cid":          rpcRsp.Cid,
			"company_name": rpcRsp.CompanyName,
			"super":        rpcRsp.Super,
		}
		str, _ := json.MarshalToString(subject)

		// 生成token
		token, err := jwtwrapper.GenToken(fmt.Sprintf("%d", rpcRsp.Uid), parser.Conf.Jwt.Issuer, str,
			time.Duration(parser.Conf.Jwt.Seconds)*time.Second)
		if err != nil {
			logger.Error("func",
				zap.String("call", "util.GenJwtToken"),
				zap.String("args", fmt.Sprintf("%d", rpcRsp.Uid)),
				zap.String("error", err.Error()))
			return errors.SystemError
		}

		resp.Data.Uid = rpcRsp.Uid
		resp.Data.Token = token
		resp.Data.User = req.User
		ctx.JSON(http.StatusOK, resp)
		return nil
	}

	// 执行任务
	httptasker.Exec(ctx, parseParamTask, handleServiceTask)
}

// token
// @Summary 刷新token
// @Description 刷新token
// @Tags 用户
// @Accept  json
// @Produce  json
// @Param token header string true "token"
// @Success 200 {object} v1.TokenResponse
// @Failure 500 {object} base.HTTPError
// @Router /api/v1/token_refresh [put]
func (c *Controller) TokenRefresh(ctx *gin.Context) {
	// 解析参数
	req := &param_v1.TokenRequest{}
	parseParamTask := func() error {
		err := util.ShouldBind(ctx, &req.Header, nil, nil, nil)
		if err != nil {
			logger.Error("func",
				zap.String("call", "util.ShouldBind"),
				zap.String("error", err.Error()))
			return errors.ParamsError
		}
		return nil
	}

	// 业务处理
	handleServiceTask := func() error {
		tokenObj, err := jwtwrapper.ParseToken(req.Token)
		if tokenObj == nil {
			return errors.TokenFailedError
		}
		if err != nil {
			switch err.(*jwt.ValidationError).Errors {
			case jwt.ValidationErrorExpired:
				if tokenObj == nil {
					return errors.TokenFailedError
				}
				if time.Now().Unix()-tokenObj.ExpiresAt > 3600 {
					return errors.TokenFailedError
				}
			default:
				return errors.TokenFailedError
			}
		}

		uid := tokenObj.Id
		subject := tokenObj.Subject
		remberPass := gjson.GetBytes([]byte(subject), "rember_password").Bool()

		// 生成token
		token, err := jwtwrapper.GenToken(uid, parser.Conf.Jwt.Issuer, subject,
			time.Duration(parser.Conf.Jwt.Seconds)*time.Second)
		if err != nil {
			logger.Error("func",
				zap.String("call", "util.GenJwtToken"),
				zap.String("args", fmt.Sprintf("%s", uid)),
				zap.String("error", err.Error()))
			return errors.SystemError
		}
		refreshTokenTime := time.Duration(24*60*60) * time.Second
		if remberPass {
			refreshTokenTime = time.Duration(7*24*60*60) * time.Second
		}
		refreshToken, err := jwtwrapper.GenToken(uid, parser.Conf.Jwt.Issuer, subject,
			refreshTokenTime)
		if err != nil {
			logger.Error("func",
				zap.String("call", "util.GenJwtToken"),
				zap.String("args", fmt.Sprintf("%s", uid)),
				zap.String("error", err.Error()))
			return errors.SystemError
		}
		resp := param_v1.TokenResponse{}
		resp.Data = token
		resp.RefreshToken = refreshToken
		ctx.JSON(http.StatusOK, resp)
		return nil
	}

	// 执行任务
	httptasker.Exec(ctx, parseParamTask, handleServiceTask)
}

//
// @Summary 获取短信验证码
// @Description 获取短信验证码
// @Tags 用户
// @Accept  json
// @Produce  json
// @Param phone query string true " "
// @Success 200 {object} v1.GetVcodeResponse
// @Failure 500 {object} base.HTTPError
// @Router /api/v1/user/vcode [get]
func (c *Controller) GetVcode(ctx *gin.Context) {
	// 解析参数
	req := &param_v1.GetVcodeRequest{}
	parseParamTask := func() error {
		err := util.ShouldBind(ctx, nil, nil, &req.GetVcodeQuery, nil)
		if err != nil {
			logger.Error("func",
				zap.String("call", "util.ShouldBind"),
				zap.String("error", err.Error()))
			return errors.ParamsError
		}
		return nil
	}

	checkPhoneTask := func() error {
		mreq := &v1.CheckPhoneRequest{Phone: req.Phone}
		_, err := pb.Company.CheckPhone(ctx, mreq)
		if err != nil {
			return err
		}
		return nil
	}

	// 业务处理
	handleServiceTask := func() error {
		// 响应数据
		resp := param_v1.GetVcodeResponse{}
		rpcReq := &v1.GetVcodeRequest{
			PhoneNumber: req.Phone,
		}

		rpcRsp, err := pb.Thirdparty.GetVcode(ctx, rpcReq)

		if err != nil {
			s, _ := json.MarshalToString(req)
			logger.Error("func",
				zap.String("call", "Thirdparty.GetVcode"),
				zap.String("params", s),
				zap.String("error", err.Error()))

			return errors.ErrorTransForm(err)
		}

		resp.Data = *rpcRsp
		ctx.JSON(http.StatusOK, resp)
		return nil
	}

	// 执行任务
	httptasker.Exec(ctx, parseParamTask, checkPhoneTask, handleServiceTask)
}

//
// @Summary 手机号登录
// @Description 手机号登录
// @Tags 用户
// @Accept  json
// @Produce  json
// @Param body body v1.LoginByPhoneBody true " "
// @Success 200 {object} v1.LoginByPhoneResponse
// @Failure 500 {object} base.HTTPError
// @Router /api/v1/user/login_by_phone [post]
func (c *Controller) LoginByPhone(ctx *gin.Context) {
	// 解析参数
	req := &param_v1.LoginByPhoneRequest{}
	loginResp := param_v1.LoginByPhoneResponse{}
	parseParamTask := func() error {
		err := util.ShouldBind(ctx, nil, nil, nil, &req.LoginByPhoneBody)
		if err != nil {
			logger.Error("func",
				zap.String("call", "util.ShouldBind"),
				zap.String("error", err.Error()))
			return errors.ParamsError
		}
		return nil
	}

	// 业务处理
	handleServiceTask := func() error {
		// 响应数据
		rpcReq := &v1.LoginByPhoneRequest{
			Phone: req.Phone,
		}

		rpcRsp, err := pb.Company.LoginByPhone(ctx, rpcReq)
		if err != nil {
			s, _ := json.MarshalToString(req)
			logger.Error("func",
				zap.String("call", "pb.Company.LoginByPhone"),
				zap.String("params", s),
				zap.String("error", err.Error()))

			return errors.ErrorTransForm(err)
		}
		subject := map[string]interface{}{
			"user_name":    rpcRsp.User,
			"cid":          rpcRsp.Cid,
			"company_name": rpcRsp.CompanyName,
			"super":        rpcRsp.Super,
		}
		str, _ := json.MarshalToString(subject)

		// 生成token
		token, err := jwtwrapper.GenToken(fmt.Sprintf("%d", rpcRsp.Uid), parser.Conf.Jwt.Issuer, str,
			time.Duration(parser.Conf.Jwt.Seconds)*time.Second)
		if err != nil {
			logger.Error("func",
				zap.String("call", "util.GenJwtToken"),
				zap.String("args", fmt.Sprintf("%d", rpcRsp.Uid)),
				zap.String("error", err.Error()))
			return errors.SystemError
		}
		if err != nil {
			logger.Error("func",
				zap.String("call", "util.GenJwtToken"),
				zap.String("args", fmt.Sprintf("%d", rpcRsp.Uid)),
				zap.String("error", err.Error()))
			return errors.SystemError
		}
		loginResp.Data.Uid = rpcRsp.Uid
		loginResp.Data.Token = token

		return nil
	}
	checkVcodeTask := func() error {
		rpcReq := &v1.CheckVcodeRequest{
			PhoneNumber: req.Phone,
			Vcode:       req.Vcode,
		}

		_, err := pb.Thirdparty.CheckVcode(ctx, rpcReq)

		if err != nil {
			s, _ := json.MarshalToString(req)
			logger.Error("func",
				zap.String("call", "pb.Thirdparty.CheckVcode"),
				zap.String("params", s),
				zap.String("error", err.Error()))

			return errors.ErrorTransForm(err)
		}
		ctx.JSON(http.StatusOK, loginResp)
		return nil
	}

	// 执行任务
	httptasker.Exec(ctx, parseParamTask, handleServiceTask, checkVcodeTask)
}

//
// @Summary 忘记密码重置密码
// @Description 忘记密码重置密码
// @Tags 用户
// @Accept  json
// @Produce  json
// @Param body body v1.ResetPasswordBody true "信息"
// @Success 200 {object} v1.ResetPasswordResponse
// @Failure 500 {object} base.HTTPError
// @Router /api/v1/user/reset_password [put]
func (c *Controller) ResetPassword(ctx *gin.Context) {
	// 解析参数
	req := &param_v1.ResetPasswordRequest{}
	parseParamTask := func() error {
		err := util.ShouldBind(ctx, nil, nil, nil, &req.ResetPasswordBody)
		if err != nil {
			logger.Error("func",
				zap.String("call", "util.ShouldBind"),
				zap.String("error", err.Error()))
			return errors.ParamsError
		}
		return nil
	}

	// 业务处理
	handleServiceTask := func() error {
		// 响应数据
		resp := param_v1.ResetPasswordResponse{}
		rpcReq := &v1.ResetPasswordRequest{
			Phone:    req.Phone,
			Vcode:    req.Vcode,
			Password: req.Password,
		}

		_, err := pb.Company.ResetPassword(ctx, rpcReq)

		if err != nil {
			s, _ := json.MarshalToString(req)
			logger.Error("func",
				zap.String("call", "pb.Company.ResetPassword"),
				zap.String("params", s),
				zap.String("error", err.Error()))

			return errors.ErrorTransForm(err)
		}
		ctx.JSON(http.StatusOK, resp)
		return nil
	}

	// 执行任务
	httptasker.Exec(ctx, parseParamTask, handleServiceTask)
}

//
// @Summary 修改密码
// @Description 修改密码
// @Tags 用户
// @Accept  json
// @Produce  json
// @Param token header string true " "
// @Param body body v1.ChangePasswordBody true "信息"
// @Success 200 {object} v1.ChangePasswordResponse
// @Failure 500 {object} base.HTTPError
// @Router /api/v1/user/change_password [put]
func (c *Controller) ChangePassword(ctx *gin.Context) {
	// 解析参数
	req := &param_v1.ChangePasswordRequest{}
	parseParamTask := func() error {
		err := util.ShouldBind(ctx, nil, nil, nil, &req.ChangePasswordBody)
		if err != nil {
			logger.Error("func",
				zap.String("call", "util.ShouldBind"),
				zap.String("error", err.Error()))
			return errors.ParamsError
		}
		return nil
	}

	// 业务处理
	handleServiceTask := func() error {
		tokenInfo, err := utils.GetJwtTokenInfo(ctx)
		if err != nil {
			return err
		}
		// 响应数据
		resp := param_v1.ChangePasswordResponse{}
		rpcReq := &v1.ChangePasswordRequest{
			Uid:         tokenInfo.Uid,
			OldPassword: req.OldPassword,
			NewPassword: req.NewPassword,
		}

		_, err = pb.Company.ChangePassword(ctx, rpcReq)

		if err != nil {
			s, _ := json.MarshalToString(req)
			logger.Error("func",
				zap.String("call", "pb.Company.ChangePassword"),
				zap.String("params", s),
				zap.String("error", err.Error()))

			return errors.ErrorTransForm(err)
		}

		ctx.JSON(http.StatusOK, resp)
		return nil
	}

	// 执行任务
	httptasker.Exec(ctx, parseParamTask, handleServiceTask)
}

//
// @Summary 添加用户
// @Description 添加用户
// @Tags 用户管理
// @Accept  json
// @Produce  json
// @Param token header string true "token"
// @Param body body v1.CompanyUserAddBody true "用户信息"
// @Success 200 {object} v1.CompanyUserAddResponse
// @Failure 500 {object} base.HTTPError
// @Router /api/v1/company_user [post]
func (c *Controller) CompanyUserAdd(ctx *gin.Context) {
	// 解析参数
	req := &param_v1.CompanyUserAddRequest{}
	parseParamTask := func() error {
		err := util.ShouldBind(ctx, &req.Header, nil, nil, &req.CompanyUserAddBody)
		if err != nil {
			logger.Error("func",
				zap.String("call", "util.ShouldBind"),
				zap.String("error", err.Error()))
			return errors.ParamsError
		}
		return nil
	}

	// 业务处理
	handleServiceTask := func() error {
		tokenInfo, err := utils.GetJwtTokenInfo(ctx)
		if err != nil {
			return err
		}
		if !tokenInfo.Super {
			return errors.PermissionError
		}
		// 响应数据
		resp := param_v1.CompanyUserAddResponse{}
		//fmt.Printf("xxxxxxxxxxxxxxxxx:garden:%d,%s\n", tokenInfo.GardenId, tokenInfo.GardenName)
		rpcReq := &v1.CompanyUserAddRequest{
			UserName: req.UserName,
			Password: req.Password,
			Phone:    req.Phone,
			Email:    req.Email,
			Cid:      tokenInfo.Cid,
			RealName: req.RealName,
		}

		rpcRsp, err := pb.Company.CompanyUserAdd(ctx, rpcReq)

		if err != nil {
			s, _ := json.MarshalToString(req)
			logger.Error("func",
				zap.String("call", "pb.Company.CompanyUserAdd"),
				zap.String("params", s),
				zap.String("error", err.Error()))

			return errors.ErrorTransForm(err)
		}
		resp.Data = *rpcRsp
		ctx.JSON(http.StatusOK, resp)

		return nil
	}

	// 执行任务
	httptasker.Exec(ctx, parseParamTask, handleServiceTask)
}

//
// @Summary 更改用户
// @Description 更改用户
// @Tags 用户管理
// @Accept  json
// @Produce  json
// @Param token header string true "token"
// @Param body body v1.CompanyUserUpdateBody true "用户信息"
// @Success 200 {object} v1.CompanyUserUpdateResponse
// @Failure 500 {object} base.HTTPError
// @Router /api/v1/company_user [put]
func (c *Controller) CompanyUserUpdate(ctx *gin.Context) {
	// 解析参数
	req := &param_v1.CompanyUserUpdateRequest{}
	parseParamTask := func() error {
		err := util.ShouldBind(ctx, &req.Header, nil, nil, &req.CompanyUserUpdateBody)
		if err != nil {
			logger.Error("func",
				zap.String("call", "util.ShouldBind"),
				zap.String("error", err.Error()))
			return errors.ParamsError
		}
		return nil
	}

	// 业务处理
	handleServiceTask := func() error {
		tokenInfo, err := utils.GetJwtTokenInfo(ctx)
		if err != nil {
			return err
		}
		if !tokenInfo.Super {
			return errors.PermissionError
		}
		// 响应数据
		resp := param_v1.CompanyUserUpdateResponse{}
		rpcReq := &v1.CompanyUserUpdateRequest{
			UserName: req.UserName,
			Password: req.Password,
			Phone:    req.Phone,
			Email:    req.Email,
			Cid:      tokenInfo.Cid,
			Id:       req.Id,
			RealName: req.RealName,
		}

		_, err = pb.Company.CompanyUserUpdate(ctx, rpcReq)

		if err != nil {
			s, _ := json.MarshalToString(req)
			logger.Error("func",
				zap.String("call", "pb.Company.CompanyUserUpdate"),
				zap.String("params", s),
				zap.String("error", err.Error()))

			return errors.ErrorTransForm(err)
		}

		ctx.JSON(http.StatusOK, resp)

		return nil
	}

	// 执行任务
	httptasker.Exec(ctx, parseParamTask, handleServiceTask)
}

//
// @Summary 删除用户
// @Description 删除用户
// @Tags 用户管理
// @Accept  json
// @Produce  json
// @Param token header string true "token"
// @Param id query int true " "
// @Success 200 {object} v1.CompanyUserDelResponse
// @Failure 500 {object} base.HTTPError
// @Router /api/v1/company_user [delete]
func (c *Controller) CompanyUserDel(ctx *gin.Context) {
	// 解析参数
	req := &param_v1.CompanyUserDelRequest{}
	parseParamTask := func() error {
		err := util.ShouldBind(ctx, &req.Header, nil, &req.CompanyUserDelQuery, nil)
		if err != nil {
			logger.Error("func",
				zap.String("call", "util.ShouldBind"),
				zap.String("error", err.Error()))
			return errors.ParamsError
		}
		return nil
	}

	// 业务处理
	handleServiceTask := func() error {
		tokenInfo, err := utils.GetJwtTokenInfo(ctx)
		if err != nil {
			return err
		}
		if !tokenInfo.Super {
			return errors.PermissionError
		}
		// 响应数据
		resp := param_v1.CompanyUserDelResponse{}
		rpcReq := &v1.CompanyUserDelRequest{
			Cid: tokenInfo.Cid,
			Id:  req.Id,
		}

		_, err = pb.Company.CompanyUserDel(ctx, rpcReq)

		if err != nil {
			s, _ := json.MarshalToString(req)
			logger.Error("func",
				zap.String("call", "pb.Company.CompanyUserDel"),
				zap.String("params", s),
				zap.String("error", err.Error()))

			return errors.ErrorTransForm(err)
		}

		ctx.JSON(http.StatusOK, resp)

		return nil
	}

	// 执行任务
	httptasker.Exec(ctx, parseParamTask, handleServiceTask)
}

//
// @Summary 用户列表
// @Description 用户列表
// @Tags 用户管理
// @Accept  json
// @Produce  json
// @Param token header string true "token"
// @Param page query int true " "
// @Param page_size query int false " "
// @Success 200 {object} v1.CompanyUserListResponse
// @Failure 500 {object} base.HTTPError
// @Router /api/v1/company_user [get]
func (c *Controller) CompanyUserList(ctx *gin.Context) {
	// 解析参数
	req := &param_v1.CompanyUserListRequest{}
	parseParamTask := func() error {
		err := util.ShouldBind(ctx, &req.Header, nil, &req.CompanyUserListQuery, nil)
		if err != nil {
			logger.Error("func",
				zap.String("call", "util.ShouldBind"),
				zap.String("error", err.Error()))
			return errors.ParamsError
		}
		return nil
	}

	// 业务处理
	handleServiceTask := func() error {
		tokenInfo, err := utils.GetJwtTokenInfo(ctx)
		if err != nil {
			return err
		}
		// 响应数据
		resp := param_v1.CompanyUserListResponse{}
		rpcReq := &v1.CompanyUserListRequest{

			Cid:      tokenInfo.Cid,
			Page:     req.Page,
			PageSize: req.PageSize,
		}

		rpcRsp, err := pb.Company.CompanyUserList(ctx, rpcReq)

		if err != nil {
			s, _ := json.MarshalToString(req)
			logger.Error("func",
				zap.String("call", "pb.Company.CompanyUserList"),
				zap.String("params", s),
				zap.String("error", err.Error()))

			return errors.ErrorTransForm(err)
		}
		if rpcRsp.List == nil {
			rpcRsp.List = make([]*v1.CompanyUserItem, 0)
		}
		resp.Data = *rpcRsp
		ctx.JSON(http.StatusOK, resp)
		return nil
	}

	// 执行任务
	httptasker.Exec(ctx, parseParamTask, handleServiceTask)
}
